﻿using AdventOfCode._2022.Models;
using AdventOfCode.Common;
using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;

namespace AdventOfCode._2022.Day11
{
    [AOC(year: 2022, day: 11)]
    public class Day11 : AOCDay
    {
        protected override AOCResponse ExecutePartA()
        {
            ExecuteMonkeyBusiness(20);
            return _response;
        }

        protected override AOCResponse ExecutePartB()
        {
            ExecuteMonkeyBusiness(10000);
            return _response;
        }

        private void ExecuteMonkeyBusiness(int rounds)
        {
            var input = SplitInputOnEmptyLine();
            var monkeys = new List<Monkey>();
            foreach (var monkey in input)
            {
                monkeys.Add(new Monkey(_request.Version, monkey.ToArray()));
            }

            long lcm = 1; //Only needed for part b
            //Target Monkey Mapping
            foreach (var monkey in monkeys)
            {
                lcm *= monkey.DivisibleNumber;
                monkey.MapTargetMonkeys(monkeys.First(x => x.MonkeyId == monkey.TargetMonkeyIds.Item1), monkeys.First(x => x.MonkeyId == monkey.TargetMonkeyIds.Item2));
            }

            //Start round
            for (int i = 0; i < rounds; i++)
            {
                foreach (var monkey in monkeys)
                {
                    monkey.Business(lcm);
                }

                if (_request.Version == AOCVersion.A)
                    monkeys.ForEach(x => Log("Round" + (i + 1) + "_M" + x.MonkeyId + ": " + string.Join(", ", x.Items)));
                else
                {
                    if ((i + 1) == 1 || (i + 1) == 20 || (i + 1) == 1000 || (i + 1) % 1000 == 0)
                    {
                        monkeys.ForEach(x => Log($"R{i + 1}Monkey{x.MonkeyId} inspected {x.ItemsInspected} items"));
                        monkeys.ForEach(x => Log("Round" + (i + 1) + "_M" + x.MonkeyId + ": " + string.Join(", ", x.Items)));
                    }
                }

            }
            monkeys.ForEach(x => Info($"Monkey{x.MonkeyId} inspected {x.ItemsInspected} items"));

            var crazyMonkeys = monkeys.OrderByDescending(x => x.ItemsInspected).ToArray();
            _response.Answer = crazyMonkeys[0].ItemsInspected * crazyMonkeys[1].ItemsInspected;
        }
    }
}
