﻿using AdventOfCode._2022.Models;
using AdventOfCode.Common;
using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AdventOfCode._2022.Day7
{
    [AOC(year: 2022, day: 7)]
    public class Day7 : AOCDay
    {
        protected override AOCResponse ExecutePartA()
        {
            var root = GenerateDirectory();

            if (!_request.IgnoreLogMessages)
                root.Print("");

            var allDirectories = root.GetAllDirectories().ToList();
            _response.Answer = allDirectories.Where(x => x.GetSize() <= 100000).Sum(x => x.GetSize());
            return _response;
        }

        protected override AOCResponse ExecutePartB()
        {
            var root = GenerateDirectory();

            var takenSpace = root.GetSize();
            var freeSpace = 70000000 - takenSpace;
            var neededSpace = 30000000 - freeSpace;

            var allDirectories = root.GetAllDirectories().ToList();
            var directoriesLargeEnough = allDirectories.Where(x => x.GetSize() >= neededSpace);
            var smallest = directoriesLargeEnough.OrderBy(x => x.GetSize()).First();
            _response.Answer = smallest.GetSize();
            return _response;
        }

        private ElfFile GenerateDirectory()
        {
            var consoleCommands = new List<ConsoleCommand>();
            var input = GetSplitInput();

            //Create our console commands
            int index = 0;
            while (index < input.Count())
            {
                int take = 1;
                while ((index + take) < input.Count() && !input[index + take].StartsWith("$"))
                {
                    take++;
                }
                consoleCommands.Add(new ConsoleCommand(input.Skip(index).Take(take)));
                index += take;
            }

            ElfFile root = null;
            ElfFile cwd = root;
            //Create our file structure
            foreach (var cmd in consoleCommands)
            {
                if (cmd.IsList())
                {
                    foreach (var file in cmd.DirectoryResult)
                    {
                        cwd.AddChild(new ElfFile((file.Size == -1), cwd, file.FileName, file.Size));
                    }
                }
                else
                {
                    if (cmd.DirectoryTarget == "/")
                    {
                        root = new ElfFile(true, null, "root");
                        root.AddChild(new ElfFile(true, null, "/"));
                        cwd = root.GetChild("/");
                    }
                    else if (cmd.DirectoryTarget == "..")
                    {
                        cwd = cwd.Parent;
                    }
                    else
                    {
                        cwd = cwd.GetChild(cmd.DirectoryTarget);
                    }
                }
            }
            return root;
        }
    }
}
