﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AdventOfCode._2022.Day9
{
    public class Rope
    {
        private RopeEnd[] _rope;
        private HashSet<string> _lastTailVisit;
        public Rope(int numberOfTails)
        {
            _rope = new RopeEnd[numberOfTails+1];
            for (int i = 0; i < numberOfTails+1; i++)
                _rope[i] = new RopeEnd();
            _lastTailVisit = new HashSet<string>();
            //Print();
        }

        public int GetUniqueVisitedByTail()
        {
            return _lastTailVisit.Count();
        }

        public void Move(Direction dir, int units)
        {
            for (int i = 0; i < units; i++)
            {
                _rope.First().Move(dir);
                for (int j = 1; j < _rope.Length; j++)
                {
                    _rope[j].Follow(_rope[j - 1]);
                }
                _lastTailVisit.Add($"{_rope.Last().X},{_rope.Last().Y}");
                //Print();
            }
        }

        //DEBUG
        private void Print()
        {
            var head = _rope.First();
            int size = 14;
            for (int x = -size; x < size; x++)
            {
                for (int y = -size; y < size; y++)
                {
                    if (head.X == y && -head.Y == x)
                        Console.Write("H ");
                    else if (_rope.Any(r => r.X == y && -r.Y == x))
                    {
                        var tail = _rope.Select((r, i) => new { Pos = i, Rope = r })
                            .First(r => r.Rope.X == y && -r.Rope.Y == x);
                        Console.Write(tail.Pos + " ");
                    }
                    else
                    {
                        Console.Write(". ");
                    }
                        
                }
                Console.WriteLine("");
            }
            //for (int x = size; x > -size; x--)
            //{
            //    for (int y = size; y > -size; y--)
            //    {
            //        if (_head.X == y && _head.Y == x)
            //            Console.Write("H ");
            //        else if (_tail.X == y && _tail.Y == x)
            //            Console.Write("T ");
            //        else
            //            Console.Write(". ");
            //    }
            //    Console.WriteLine("");
            //}
            Console.WriteLine();
        }
    }

    class RopeEnd
    {
        public int X { get; set; }
        public int Y { get; set; }

        public RopeEnd()
        {
            X = 0;
            Y = 0;
        }

        public void Move(Direction dir)
        {
            switch (dir)
            {
                case Direction.U:
                    Y++;
                    break;
                case Direction.D:
                    Y--;
                    break;
                case Direction.L:
                    X--;
                    break;
                case Direction.R:
                    X++;
                    break;
                default:
                    break;
            }
        }

        public void Follow(RopeEnd head)
        {
            var distance = Math.Floor(Math.Sqrt((Math.Pow(head.X - X, 2) + Math.Pow(head.Y - Y, 2))));
            if (distance <= 1) return;
            //Diagonal Move
            if (head.X != X && head.Y != Y)
            {
                if (head.X > X && head.Y > Y)
                {
                    X++; Y++;
                }
                else if (head.X > X && head.Y < Y)
                {
                    X++; Y--;
                }
                else if (head.X < X && head.Y > Y)
                {
                    X--; Y++;
                }
                else if (head.X < X && head.Y < Y)
                {
                    X--; Y--;
                }
                else
                {
                    Console.WriteLine("Diag shouldn't happen");
                }
            }
            else if (head.X == X && head.Y != Y)
            {
                if (head.Y > Y) Y++; //Head is above
                else Y--; //Head below
            }
            else if (head.Y == Y && head.X != X)
            {
                if (head.X > X) X++; //Head is to the right of tail
                else X--; //Head is to the left
            }
            else
            {
                Console.WriteLine("This shouldn't happen?");
            }

        }
    }
}
